@@ -5,12 +5,16 @@ DEFAULT_MODEL = "SD-1.5"
 DEFAULT_SAMPLER = "k_dpmpp_2m"
 
 DEFAULT_NEGATIVE_PROMPT = (
-    "weird eyes, smudged face, blurred face, poorly drawn face, mutation, mutilation, cloned face, strange mouth, "
-    "Ugly, duplication, duplicates, mutilation, deformed, mutilated, mutation, twisted body, disfigured, bad anatomy, "
-    "poorly drawn hands, extra limbs, malformed limbs, missing arms, extra arms, missing legs, extra legs, mutated hands, "
-    "extra hands, fused fingers, missing fingers, extra fingers, long neck, small head, closed eyes, rolling eyes, "
-    "grainy, blurred, blurry, writing, calligraphy, signature, text, watermark, bad art, "
-    "out of frame, extra fingers, mutated hands, porn, child abuse, horrific images, dead bodies"
+    "(weird eyes:1.5), smudged face, (blurred face:1.5), (poorly drawn face:1.5), mutation, mutilation, cloned face, (strange mouth:1.5), (duplicated face:2.0), "
+    "(Ugly:1.5), *duplication:1.5), duplicates, mutilation, deformed, mutilated, (mutation:1.5), (twisted body:1.5), (disfigured:1.5), (bad anatomy:1.5), "
+    "(poorly drawn hands:1.5), (extra limbs:1.5), (malformed limbs:1.5), (missing arms:1.5), (extra arms:1.5), (missing legs:1.5), (extra legs:1.5), (mutated hands:1.5), "
+    "(extra hands:1.5), (fused fingers:1.5), (missing fingers:1.5), extra fingers, long neck, small head, closed eyes, rolling eyes, "
+    "grainy, blurred, blurry, writing, calligraphy, text, (watermark:1.5), bad art, "
+    "(out of frame:1.5), (extra fingers:1.5), (mutated hands:1.5), (porn:1.5), (child abuse:1.5), (horrific images:1.5), (dead bodies:1.5)",
+    "(watermark:1.5), child, lowres, bad anatomy, bad hands, text, error, (missing fingers:1.5), extra digit, fewer digits, cropped,"
+    "worst quality, (low quality:1.5), normal quality, jpeg artifacts, (signature:1.5), watermark, username, blurry, (bad feet:1.5), bad lighting,"
+    "bad coloring, bad perspective, (bad focus:1.5), bad design, bad eyes, bad hair, bad nose, bad mouth",
+    "(nude:1.5), (naked:1.5), (paedophile:2.0), (pedo:2.0)"
 )
 
 SPLITMEM_ENABLED = False
@@ -167,7 +171,7 @@ MODEL_CONFIGS = [
     ModelConfig(
         short_name="RPG",
         config_path="configs/stable-diffusion-v1.yaml",
-        weights_url="https://huggingface.co/Anashel/rpg/blob/resolve/RPG-V4-Model-Download/RPG-v4.safetensors",
+        weights_url="https://huggingface.co/Anashel/rpg/resolve/main/RPG-V4-Model-Download/RPG-v4.safetensors",
         default_image_size=512,
         alias="rpg"
     ),
