@@ -2,12 +2,12 @@ from functools import lru_cache
 
 import numpy as np
 import torch
-from basicsr.archs.rrdbnet_arch import RRDBNet
 from PIL import Image
-from realesrgan import RealESRGANer
 
 from classy_imaginary.model_manager import get_cached_url_path
 from classy_imaginary.utils import get_device
+from classy_imaginary.vendored.basicsr.rrdbnet_arch import RRDBNet
+from classy_imaginary.vendored.realesrgan import RealESRGANer
 
 
 @lru_cache()
@@ -17,7 +17,7 @@ def realesrgan_upsampler():
     )
     url = "https://github.com/xinntao/Real-ESRGAN/releases/download/v0.1.0/RealESRGAN_x4plus.pth"
     model_path = get_cached_url_path(url)
-    upsampler = RealESRGANer(scale=4, model_path=model_path, model=model, tile=0)
+    upsampler = RealESRGANer(scale=4, model_path=model_path, model=model, tile=512)
 
     device = get_device()
     if "mps" in device:
